/*
**> In this program, we simulate local volatility process and price options using MC
** 
*/

new;
library option,pgraph;
OptionSet;

cls;

loadm databuf = Dupire-ImpVol;

Smile_K = vread(databuf,"K");
Smile_T = vread(databuf,"T");
Smile_ImpVol = vread(databuf,"ImpVol");

S0 = 100;
b = 0.05;
r = 0.05;

FileName = "Dupire11";
call LocalVol_Init(Smile_K,Smile_ImpVol,Smile_T,0,200,201,1,201,S0,b,r,FileName);

t = seqa(0,0.01,101);
Ns = 10;
x = LocalVol_simulate_SDE(t,Ns);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10;
  xlabel("\216t");
  ylabel("\216S(t)");
  xtics(0,1,0.10,1);
  title("\216Ten simulated paths of the local volatility process");
  graphprt("-c=1 -cf=dupire13.eps");
  xy(t,x);

Ns = 1000;  // Use a larger value to check the convergence
Tau = 1.0;
K = 100;
L = 90;
H = 115;

t = seqa(0,Tau/100,101); nT = rows(t);

let string OptionType[10,1] = "call" "put" "doc" "dop" "uoc" "uop" "koc" "kop" "bc" "bp";
nIters = rows(OptionType);

__output = 0;

i = 1;
do until i > nIters;

  print /flush "--------------------------------------------------------------";

  {tmp_S0,tmp_Tau,C} = LocalVol_Backward_PDE_Solve(OptionType[i],K,L,H,tau,0.5);
  str = upper(OptionType[i]) $+ "(PDE)" $+ ftos(C," = %lf",5,4);
  print /flush str;

  S = LocalVol_simulate_SDE(t,Ns);
  e = ones(Ns,1);
  j = 1;
  do until j > nT;
    e = e .and _LocalVol_restrictions(0,S[j,.]');
    j = j + 1;
  endo;

  ST = S[Nt,.]';
  payout = e .* _LocalVol_payoff(ST,K);

  C = exp(-r*tau)*meanc(payout);
  str = upper(OptionType[i]) $+ "(MC) " $+ ftos(C," = %lf",5,4);
  print /flush str;

  i = i + 1;
endo;

  



