/*
**> In this program, we compute the local volatility from the implied volatility
**  In particular, we show the impact of the maturity.
** 
*/

new;
library option,pgraph;
OptionSet;

cls;

let Smile_K      = 80   85   90   95   100  105  110   115  120;
let Smile_ImpVol = 0.23 0.22 0.21 0.20 0.19 0.20 0.21 0.22 0.23;

S0 = 100;
r = 0.05;
b = 0.05;

declare matrix p_smoothing;

proc SigmaSurface(t,K);
 retp(fspline(csspline(Smile_K,Smile_ImpVol,1,p_smoothing),K));   /* Cubic Spline Smoothing */
endp;

p_smoothing = 0.10;
K = seqa(70,1,61);

T = 1/52;
ImpVol1 = SigmaSurface(t,K);
LocalVol1 = ImpVol_to_LocalVol(&SigmaSurface,0,0,0,T,K,S0,b);

T = 1/12;
ImpVol2 = SigmaSurface(t,K);
LocalVol2 = ImpVol_to_LocalVol(&SigmaSurface,0,0,0,T,K,S0,b);

T = 6/12;
ImpVol3 = SigmaSurface(t,K);
LocalVol3 = ImpVol_to_LocalVol(&SigmaSurface,0,0,0,T,K,S0,b);

T = 1;
ImpVol4 = SigmaSurface(t,K);
LocalVol4 = ImpVol_to_LocalVol(&SigmaSurface,0,0,0,T,K,S0,b);

graphset;
  begwind;
  window(2,2,0);

    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 6|1; _plwidth = 10; _pcolor = 9|12;
    _paxht = 0.25; _pnumht = 0.25; _ptitlht = 0.30;
    xtics(70,130,10,2);
    ytics(10,30,5,2);
    xlabel("\216Strike");
  
  setwind(1);
    title("\216T = 1 Week");
    xy(K,100*(ImpVol1~LocalVol1));

  setwind(2);
    title("\216T = 1 Month");
    xy(K,100*(ImpVol2~LocalVol2));

  setwind(3);
    title("\216T = 6 Months");
    xy(K,100*(ImpVol3~LocalVol3));

  setwind(4);
    title("\216T = 1 Year");
    _plegstr = "Implied Vol\000Local Vol";
    _plegctl = {2 5 5.5 2};
    xy(K,100*(ImpVol4~LocalVol4));

  endwind;