/*
**> Comprarison of computational times between matrix_simulate_mGBM and array_simulate_mGBM
**
*/

new;
library option,pgraph;

cls;

let x0 = 100 50 75;            
let mu = 0.05 0.07 0.08;
let sigma = 0.15 0.25 0.05;

let rho = {1,
           0.5, 1,
           0.25, 0.15, 1};

rho = xpnd(rho);  // transforms the vector 'vech(rho)' into a correlation matrix

dt = 365;
t = seqa(0,1/dt,5*dt+1);

/* 
** We first verify that the procedures array_simulate_GBM
** and matrix_simulate_GBM give the same result when
** the number of simulated paths is 1.
*/

rndseed 123;
x = array_simulate_mGBM(x0,mu,sigma,rho,t,1);
x = getmatrix(atranspose(x,2|1|3),1);

rndseed 123;
y = matrix_simulate_mGBM(x0,mu,sigma,rho,t);

err = maxc(maxc(abs(x-y)));
print ftos(err,"Maximal absolute error = %lE",10,5);

/* 
** We compare now the computational times
*/

let ns = 1 2 3 10 100 500;

print;
print "Computational time for matrix_simulate_mGBM";

j = 1;
do until j > rows(ns);

  t0 = hsec;
  for i (1,ns[j],1);
    y = matrix_simulate_mGBM(x0,mu,sigma,rho,t);
  endfor;
  print /flush ftos(ns[j],"for %lf simulations",1,0)$+ftos((hsec-t0)/100," = %lf seconds",5,2);

  j = j + 1;
endo;

print;
print "Computational time for array_simulate_mGBM";

j = 1;
do until j > rows(ns);

  t0 = hsec;
  x = array_simulate_mGBM(x0,mu,sigma,rho,t,ns[j]);
  print /flush ftos(ns[j],"for %lf simulations",1,0)$+ftos((hsec-t0)/100," = %lf seconds",5,2);
  j = j + 1;
endo;

