/*
**> Pricing the average of Best-Of in Time Across Assets using the Black-Scholes model
**
*/

new;
library option,pgraph;

cls;

let x0 = 100 100 100;            
let mu = 0.05 0.06 0.07;
let sigma = 0.15 0.25 0.20;                  // Implied Volatility

// Correlation Matrix between Asset returns

let rho = {1,
           0.5, 1,
           0.25, 0.15, 1};
rho = xpnd(rho); 

// First, we have to correct the drift
r = 0.05;                                    // risk-free interest rate
mu = ones(rows(mu),1) * r;

// Caracteristics of the option
t = seqa(0,0.25,9);                          // 8 Fixing Dates : Q1, Q2, Q3, Q4, Q5, Q6, Q7, Q8
tau = t[9];                                  // Maturity of the option = 2 Years
Notional = 1000;                             // Notional of the option
K = 0.10;                                    // Strike of the option = 10%
fn call_payoff(S,K) = (S - K) .* (S .>= K);  // call option
fn put_payoff(S,K) = (K - S) .* (S .<= K);   // put option

Ns = 1000000;  // Number of simulations
x = array_simulate_mGBM(x0,mu,sigma,rho,t,Ns);

Nx = rows(x0);
Nt = rows(t);
BestOf = zeros(Ns,Nx);

i = 1;
do until i > Nx;                             // for each asset
  data = mGBM_getAsset(x,i);
  tmp = data[.,2:Nt] ./ data[.,1:Nt-1] - 1;  // we compute the quaterly returns for the 2 years period
  BestOf[.,i] = maxc(tmp');                  // we compute the maximum of the quaterly returns
  i = i + 1;
endo;

// BestOf is a Ns x Nx matrix where BestOf(i,j) indicates the Best-Of the quaterly returns of the asset j for the simulation i

MeanBestOf = meanc(BestOf');                 // Average of Best-Of in Time Across Assets

C = exp(-r*tau) * meanc( call_payoff(MeanBestOf,K) );
P = exp(-r*tau) * meanc( put_payoff(MeanBestOf,K) );

print ftos(C,"Best-Of Call Option Price = %lf",3,2);
print ftos(P,"Best-Of Put Option Price = %lf",3,2);

