/*
**> Comparison of the KOC prices obtained with the Rubinstein-Reiner formula and the PDE solver
**
*/

new;
library option,pgraph;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

S0 = 100;
K = 100;
sigma = 0.10;
tau = 1;
r = 0.05;
b = 0.05;

let Barrier[1,3] = 0 80 120; // KOC option with L = 80 and H = 120 for all the period

Smin = 50;
Smax = 150;
tmin = 0;
tmax = tau;

nS = 2001;
nt = 10001;

_PDE_SaveLastIter = 1;
_option_type = "call";

call PartialBarrier_PDE(Smin,Smax,nS,tmin,tmax,nt,K,sigma,b,r,Barrier,"tmp",0.5);

t = PDE_readFile("tmp","t");
x = PDE_readFile("tmp","x");
U = PDE_readFile("tmp","t"|t);

S = seqa(70,1,61);
indx = FindIndex(x,S);
x = x[indx];
U = U[indx];

L = Barrier[1,2];
H = Barrier[1,3];
KOP = EuropeanBS_KOC(miss(S,0),K,sigma,t,b,r,t,L,H);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|3; _plwidth = 10;
  title("\216Comparison of the KOC prices obtained with\L\216the Rubinstein-Reiner formula and the PDE solver");
  xtics(70,130,10,2);
  xlabel("\216S]0[");
  ylabel("\216Call Option Price");
  _plegstr = "PDE\000Rubinstein-Reiner";
  _plegctl = {2 5 5 2};
  xy(S,U~KOP);

